/*
 *	@(#)CubicSplineCurve.java 1.3 98/11/24 17:28:03
 *
 * Copyright (c) 1996-1998 Sun Microsystems, Inc. All Rights Reserved.
 *
 * Sun grants you ("Licensee") a non-exclusive, royalty free, license to use,
 * modify and redistribute this software in source and binary code form,
 * provided that i) this copyright notice and license appear on all copies of
 * the software; and ii) Licensee does not utilize the software in a manner
 * which is disparaging to Sun.
 *
 * This software is provided "AS IS," without a warranty of any kind. ALL
 * EXPRESS OR IMPLIED CONDITIONS, REPRESENTATIONS AND WARRANTIES, INCLUDING ANY
 * IMPLIED WARRANTY OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR
 * NON-INFRINGEMENT, ARE HEREBY EXCLUDED. SUN AND ITS LICENSORS SHALL NOT BE
 * LIABLE FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING
 * OR DISTRIBUTING THE SOFTWARE OR ITS DERIVATIVES. IN NO EVENT WILL SUN OR ITS
 * LICENSORS BE LIABLE FOR ANY LOST REVENUE, PROFIT OR DATA, OR FOR DIRECT,
 * INDIRECT, SPECIAL, CONSEQUENTIAL, INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER
 * CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY, ARISING OUT OF THE USE OF
 * OR INABILITY TO USE SOFTWARE, EVEN IF SUN HAS BEEN ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGES.
 *
 * This software is not designed or intended for use in on-line control of
 * aircraft, air traffic, aircraft navigation or aircraft communications; or in
 * the design, construction, operation or maintenance of any nuclear
 * facility. Licensee represents and warrants that it will not use or
 * redistribute the Software for such purposes.
 */

package com.sun.j3d.utils.behaviors.interpolators;

import javax.media.j3d.*;
import java.util.*;
import javax.vecmath.*;
import com.sun.j3d.utils.internal.J3dUtilsI18N;

/**
 * CubicSplineCurve is a container class that holds a number of 
 * cubicSplineSegments 
 * 
 * @version     1.3, 98/11/24 17:28:03
 * @author  	Vijay Sarathy	
 */

public class CubicSplineCurve {
  
    private float                  totalCurveLength; 
    private CubicSplineSegment[]   cubicSplineSegment;
    public  int                    numSegments;
 

    // default constructor 
    CubicSplineCurve () {
        numSegments = 0;
        totalCurveLength = 0f;
    }

    /**
     * This method takes a list of key frames and creates spline segments
     * from it. It requires at least four key frames to be passed to it.
     * Given n key frames, it creates n-3 CubicSplineSegments.
     * @param the list of key frames that specify the motion path 
     */

    CubicSplineCurve (TCBKeyFrame keys[]) {

        int keyLength = keys.length;
        // Require at least 4 key frames for cubic spline curve 
        if (keyLength < 4)
          throw new IllegalArgumentException(J3dUtilsI18N.getString("CubicSplineCurve0"));
        
        numSegments = keyLength - 3;
        this.cubicSplineSegment = new CubicSplineSegment[numSegments];

        // intialize and calculate coefficients for each segment 
        int k0 = 0; int k1 = 1; int k2 = 2; int k3 = 3;
        for (; k0 < numSegments; k0++, k1++, k2++, k3++) {
           this.cubicSplineSegment[k0] = new CubicSplineSegment
                                (keys[k0], keys[k1], keys[k2], keys[k3]);
        }

        // compute total curve length
        computeTotalCurveLength ();
    }

    /**
     * This method takes a list of spline segments creates the 
     * CubicSplineCurve. 
     * @param the list of segments that comprise the complete motion path
     */

    CubicSplineCurve (CubicSplineSegment s[]) {

        cubicSplineSegment = new CubicSplineSegment[s.length];
        numSegments = cubicSplineSegment.length;
        for (int i = 0; i < numSegments; i++) {
           this.cubicSplineSegment[i] = s[i];
        }

        // compute total curve length
        computeTotalCurveLength ();
    }

    /**
     * This method takes a list of spline segments to replace the existing 
     * set of CubicSplineSegments that comprise the current CubicSplineCurve
     * motion path. 
     * @param the list of segments that comprise the complete motion path
     */

    public void setSegments (CubicSplineSegment s[]) {

        cubicSplineSegment = new CubicSplineSegment[s.length];
        numSegments = cubicSplineSegment.length;
        for (int i = 0; i < numSegments; i++) {
           this.cubicSplineSegment[i] = s[i];
        }

        // compute total curve length
        computeTotalCurveLength ();
    }

    /**
     * This method returns the CubicSplineSegments pointed to by index 
     * @param the index of the CubicSplineSegment required 
     * @return the CubicSplineSegment pointed to by index
     */
    public CubicSplineSegment getSegment (int index) {

        return this.cubicSplineSegment[index];

    } 
    

    // computes the total length of the curve 
    private void computeTotalCurveLength () {

        totalCurveLength = 0f;
        for (int i = 0; i < numSegments; i++) {
           totalCurveLength += cubicSplineSegment[i].length;
        } 

    }

    /**
     * This method returns the total length of the entire CubicSplineCurve 
     * motion path.
     *
     * @return the length of the CubicSplineCurve motion path 
     */

    public float getTotalCurveLength () {

        return this.totalCurveLength;

    }
  
}

